@@ -1,7 +1,7 @@
 /*
  * weechat-ruby.c - ruby plugin for WeeChat
  *
- * Copyright (C) 2003-2021 Sébastien Helleu <flashcode@flashtux.org>
+ * Copyright (C) 2003-2023 Sébastien Helleu <flashcode@flashtux.org>
  * Copyright (C) 2005-2007 Emmanuel Bouthenot <kolter@openics.org>
  *
  * This file is part of WeeChat, the extensible chat client.
@@ -30,6 +30,7 @@
 #include <ruby/version.h>
 #endif
 
+#include <signal.h>
 #include <stdarg.h>
 #include <sys/types.h>
 #include <sys/stat.h>
@@ -61,7 +62,7 @@ WEECHAT_PLUGIN_DESCRIPTION(N_("Support of ruby scripts"));
 WEECHAT_PLUGIN_AUTHOR("Sébastien Helleu <flashcode@flashtux.org>");
 WEECHAT_PLUGIN_VERSION(WEECHAT_VERSION);
 WEECHAT_PLUGIN_LICENSE(WEECHAT_LICENSE);
-WEECHAT_PLUGIN_PRIORITY(4000);
+WEECHAT_PLUGIN_PRIORITY(RUBY_PLUGIN_PRIORITY);
 
 struct t_weechat_plugin *weechat_ruby_plugin = NULL;
 
@@ -265,24 +266,26 @@ rb_protect_funcall (VALUE recv, ID mid, int *state, int argc, VALUE *argv)
 int
 weechat_ruby_print_exception (VALUE err)
 {
-    VALUE backtrace, tmp1, tmp2, tmp3;
+    VALUE backtrace, message, class, class_name, tmp3;
     int i;
     int ruby_error;
-    char* line;
-    char* cline;
-    char* err_msg;
-    char* err_class;
+    char *line, **cline, *err_msg, *err_class;
 
     backtrace = rb_protect_funcall (err, rb_intern ("backtrace"),
                                     &ruby_error, 0, NULL);
 
-    tmp1 = rb_protect_funcall(err, rb_intern ("message"), &ruby_error, 0, NULL);
-    err_msg = StringValueCStr (tmp1);
+    message = rb_protect_funcall(err, rb_intern ("message"), &ruby_error, 0, NULL);
+    err_msg = StringValueCStr (message);
 
-    tmp2 = rb_protect_funcall (rb_protect_funcall (err, rb_intern ("class"),
-                                                   &ruby_error, 0, NULL),
-                               rb_intern ("name"), &ruby_error, 0, NULL);
-    err_class = StringValuePtr (tmp2);
+    err_class = NULL;
+    class = rb_protect_funcall (err, rb_intern ("singleton_class"),
+                                &ruby_error, 0, NULL);
+    if (class != Qnil)
+    {
+        class_name = rb_protect_funcall (class, rb_intern ("to_s"),
+                                         &ruby_error, 0, NULL);
+        err_class = StringValuePtr (class_name);
+    }
 
     if (strcmp (err_class, "SyntaxError") == 0)
     {
@@ -294,46 +297,34 @@ weechat_ruby_print_exception (VALUE err)
     }
     else
     {
+        cline = weechat_string_dyn_alloc (256);
         for (i = 0; i < RARRAY_LEN(backtrace); i++)
         {
             line = StringValuePtr (RARRAY_PTR(backtrace)[i]);
-            cline = NULL;
+            weechat_string_dyn_copy (cline, NULL);
             if (i == 0)
             {
-                cline = (char *)calloc (strlen (line) + 2 + strlen (err_msg) +
-                                        3 + strlen (err_class) + 1,
-                                        sizeof (char));
-                if (cline)
+                weechat_string_dyn_concat (cline, line, -1);
+                weechat_string_dyn_concat (cline, ": ", -1);
+                weechat_string_dyn_concat (cline, err_msg, -1);
+                if (err_class)
                 {
-                    strcat (cline, line);
-                    strcat (cline, ": ");
-                    strcat (cline, err_msg);
-                    strcat (cline, " (");
-                    strcat (cline, err_class);
-                    strcat (cline, ")");
+                    weechat_string_dyn_concat (cline, " (", -1);
+                    weechat_string_dyn_concat (cline, err_class, -1);
+                    weechat_string_dyn_concat (cline, ")", -1);
                 }
             }
             else
             {
-                cline = (char *)calloc (strlen (line) + strlen ("     from ") + 1,
-                                        sizeof (char));
-                if (cline)
-                {
-                    strcat (cline, "     from ");
-                    strcat (cline, line);
-                }
+                weechat_string_dyn_concat (cline, "     from ", -1);
+                weechat_string_dyn_concat (cline, line, -1);
             }
-            if (cline)
-            {
-                weechat_printf (NULL,
-                                weechat_gettext ("%s%s: error: %s"),
-                                weechat_prefix ("error"), RUBY_PLUGIN_NAME,
-                                cline);
-            }
-
-            if (cline)
-                free (cline);
+            weechat_printf (NULL,
+                            weechat_gettext ("%s%s: error: %s"),
+                            weechat_prefix ("error"), RUBY_PLUGIN_NAME,
+                            *cline);
         }
+        weechat_string_dyn_free (cline, 1);
     }
 
     return 0;
@@ -480,8 +471,11 @@ weechat_ruby_exec (struct t_plugin_script *script,
         {
             switch (format[i])
             {
-                case 's': /* string */
-                    argv2[i] = rb_str_new2 ((char *)argv[i]);
+                case 's': /* string or null */
+                    if (argv[i])
+                        argv2[i] = rb_str_new2 ((char *)argv[i]);
+                    else
+                        argv2[i] = Qnil;
                     break;
                 case 'i': /* integer */
                     argv2[i] = INT2FIX (*((int *)argv[i]));
@@ -734,10 +728,14 @@ weechat_ruby_load (const char *filename, const char *code)
 void
 weechat_ruby_load_cb (void *data, const char *filename)
 {
+    const char *pos_dot;
+
     /* make C compiler happy */
     (void) data;
 
-    weechat_ruby_load (filename, NULL);
+    pos_dot = strrchr (filename, '.');
+    if (pos_dot && (strcmp (pos_dot, ".rb") == 0))
+        weechat_ruby_load (filename, NULL);
 }
 
 /*
@@ -796,7 +794,7 @@ weechat_ruby_unload_name (const char *name)
 {
     struct t_plugin_script *ptr_script;
 
-    ptr_script = plugin_script_search (weechat_ruby_plugin, ruby_scripts, name);
+    ptr_script = plugin_script_search (ruby_scripts, name);
     if (ptr_script)
     {
         weechat_ruby_unload (ptr_script);
@@ -825,7 +823,7 @@ weechat_ruby_reload_name (const char *name)
     struct t_plugin_script *ptr_script;
     char *filename;
 
-    ptr_script = plugin_script_search (weechat_ruby_plugin, ruby_scripts, name);
+    ptr_script = plugin_script_search (ruby_scripts, name);
     if (ptr_script)
     {
         filename = strdup (ptr_script->filename);
@@ -945,30 +943,30 @@ weechat_ruby_command_cb (const void *pointer, void *data,
     }
     else if (argc == 2)
     {
-        if (weechat_strcasecmp (argv[1], "list") == 0)
+        if (weechat_strcmp (argv[1], "list") == 0)
         {
             plugin_script_display_list (weechat_ruby_plugin, ruby_scripts,
                                         NULL, 0);
         }
-        else if (weechat_strcasecmp (argv[1], "listfull") == 0)
+        else if (weechat_strcmp (argv[1], "listfull") == 0)
         {
             plugin_script_display_list (weechat_ruby_plugin, ruby_scripts,
                                         NULL, 1);
         }
-        else if (weechat_strcasecmp (argv[1], "autoload") == 0)
+        else if (weechat_strcmp (argv[1], "autoload") == 0)
         {
             plugin_script_auto_load (weechat_ruby_plugin, &weechat_ruby_load_cb);
         }
-        else if (weechat_strcasecmp (argv[1], "reload") == 0)
+        else if (weechat_strcmp (argv[1], "reload") == 0)
         {
             weechat_ruby_unload_all ();
             plugin_script_auto_load (weechat_ruby_plugin, &weechat_ruby_load_cb);
         }
-        else if (weechat_strcasecmp (argv[1], "unload") == 0)
+        else if (weechat_strcmp (argv[1], "unload") == 0)
         {
             weechat_ruby_unload_all ();
         }
-        else if (weechat_strcasecmp (argv[1], "version") == 0)
+        else if (weechat_strcmp (argv[1], "version") == 0)
         {
             plugin_script_display_interpreter (weechat_ruby_plugin, 0);
         }
@@ -977,19 +975,19 @@ weechat_ruby_command_cb (const void *pointer, void *data,
     }
     else
     {
-        if (weechat_strcasecmp (argv[1], "list") == 0)
+        if (weechat_strcmp (argv[1], "list") == 0)
         {
             plugin_script_display_list (weechat_ruby_plugin, ruby_scripts,
                                         argv_eol[2], 0);
         }
-        else if (weechat_strcasecmp (argv[1], "listfull") == 0)
+        else if (weechat_strcmp (argv[1], "listfull") == 0)
         {
             plugin_script_display_list (weechat_ruby_plugin, ruby_scripts,
                                         argv_eol[2], 1);
         }
-        else if ((weechat_strcasecmp (argv[1], "load") == 0)
-                 || (weechat_strcasecmp (argv[1], "reload") == 0)
-                 || (weechat_strcasecmp (argv[1], "unload") == 0))
+        else if ((weechat_strcmp (argv[1], "load") == 0)
+                 || (weechat_strcmp (argv[1], "reload") == 0)
+                 || (weechat_strcmp (argv[1], "unload") == 0))
         {
             ptr_name = argv_eol[2];
             if (strncmp (ptr_name, "-q ", 3) == 0)
@@ -1001,7 +999,7 @@ weechat_ruby_command_cb (const void *pointer, void *data,
                     ptr_name++;
                 }
             }
-            if (weechat_strcasecmp (argv[1], "load") == 0)
+            if (weechat_strcmp (argv[1], "load") == 0)
             {
                 /* load ruby script */
                 path_script = plugin_script_search_path (weechat_ruby_plugin,
@@ -1011,19 +1009,19 @@ weechat_ruby_command_cb (const void *pointer, void *data,
                 if (path_script)
                     free (path_script);
             }
-            else if (weechat_strcasecmp (argv[1], "reload") == 0)
+            else if (weechat_strcmp (argv[1], "reload") == 0)
             {
                 /* reload one ruby script */
                 weechat_ruby_reload_name (ptr_name);
             }
-            else if (weechat_strcasecmp (argv[1], "unload") == 0)
+            else if (weechat_strcmp (argv[1], "unload") == 0)
             {
                 /* unload ruby script */
                 weechat_ruby_unload_name (ptr_name);
             }
             ruby_quiet = 0;
         }
-        else if (weechat_strcasecmp (argv[1], "eval") == 0)
+        else if (weechat_strcmp (argv[1], "eval") == 0)
         {
             send_to_buffer_as_input = 0;
             exec_commands = 0;
@@ -1140,7 +1138,7 @@ weechat_ruby_infolist_cb (const void *pointer, void *data,
     if (!infolist_name || !infolist_name[0])
         return NULL;
 
-    if (weechat_strcasecmp (infolist_name, "ruby_script") == 0)
+    if (strcmp (infolist_name, "ruby_script") == 0)
     {
         return plugin_script_infolist_list_scripts (weechat_ruby_plugin,
                                                     ruby_scripts, obj_pointer,
@@ -1165,8 +1163,7 @@ weechat_ruby_signal_debug_dump_cb (const void *pointer, void *data,
     (void) signal;
     (void) type_data;
 
-    if (!signal_data
-        || (weechat_strcasecmp ((char *)signal_data, RUBY_PLUGIN_NAME) == 0))
+    if (!signal_data || (strcmp ((char *)signal_data, RUBY_PLUGIN_NAME) == 0))
     {
         plugin_script_print_log (weechat_ruby_plugin, ruby_scripts);
     }
@@ -1333,6 +1330,10 @@ weechat_plugin_init (struct t_weechat_plugin *plugin, int argc, char *argv[])
         "end\n"
     };
 
+    /* make C compiler happy */
+    (void) argc;
+    (void) argv;
+
     weechat_ruby_plugin = plugin;
 
     /* set interpreter name and version */
@@ -1406,7 +1407,7 @@ weechat_plugin_init (struct t_weechat_plugin *plugin, int argc, char *argv[])
     ruby_data.unload_all = &weechat_ruby_unload_all;
 
     ruby_quiet = 1;
-    plugin_script_init (weechat_ruby_plugin, argc, argv, &ruby_data);
+    plugin_script_init (weechat_ruby_plugin, &ruby_data);
     ruby_quiet = 0;
 
     plugin_script_display_short_list (weechat_ruby_plugin,
@@ -1434,6 +1435,7 @@ weechat_plugin_end (struct t_weechat_plugin *plugin)
     ruby_quiet = 0;
 
     ruby_cleanup (0);
+    signal (SIGCHLD, SIG_DFL);
 
     /* free some data */
     if (ruby_action_install_list)
